/*
 * Copyright 2021 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.nettool.proxy;

import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.eclipse.jetty.http.HttpURI;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletHandler;
import org.eclipse.jetty.servlets.ProxyServlet;

public class LimitingProxy extends ProxyServlet {
	private static int listenport;
	private static String forwardurl_prefix;
	private static String targetpath_prefix;
	private static int concurrencylimit;
	private static String busyurl;
	private static int verbose;
	private static int inprogress = 0;
	private static Object lck = new Object();
    @Override
    public void service(ServletRequest req, ServletResponse res) throws ServletException, IOException {
	    URL target = new URL(forwardurl_prefix);
	    if(target.getPort() == target.getDefaultPort() || target.getPort() <= 0) {
	        ((Request)req).getConnection().getRequestFields().put("Host", target.getHost());
	    } else {
	        ((Request)req).getConnection().getRequestFields().put("Host", target.getHost() + ":" + target.getPort());
	    }
    	if(((HttpServletRequest)req).getPathInfo().startsWith(targetpath_prefix)) {
        	synchronized(lck) {
        		inprogress++;
        	}
        	super.service(req, res);
        	synchronized(lck) {
        		inprogress--;
        	}
    	} else {
        	super.service(req, res);
    	}
    }
	@Override
	protected HttpURI proxyHttpURI(final String scheme, final String serverName, int serverPort, final String path) throws MalformedURLException {
		if(path.startsWith(targetpath_prefix)) {
			boolean busy;
			synchronized(lck) {
				busy = inprogress > concurrencylimit;
				if(verbose >= 2) {
					System.out.printf("%s : %s / %s -> %s\n", path, inprogress, concurrencylimit, busy ? "busy" : "fine");
				}
			}
			if(!busy) { 
				return new HttpURI(forwardurl_prefix + path);
			} else {
				if(verbose == 1) {
					synchronized(lck) {
						System.out.printf("%s busy\n", path);
					}
				}
				return new HttpURI(busyurl);
			}
		} else {
			if(verbose >= 3) {
				synchronized(lck) {
					System.out.printf("%s skipped\n", path);
				}
			}
			return new HttpURI(forwardurl_prefix + path);
		}
	}
	public static void main(String[] args) throws Exception {
		if(args.length == 6) {
			listenport = Integer.parseInt(args[0]);
			forwardurl_prefix = args[1];
			targetpath_prefix = args[2];
			concurrencylimit = Integer.parseInt(args[3]);
			busyurl = args[4];
			verbose = Integer.parseInt(args[5]);
			Server srv = new Server(listenport);
	        ServletHandler proxy = new ServletHandler();
	        proxy.addServletWithMapping(LimitingProxy.class, "/*");
	        srv.setHandler(proxy);
			srv.start();
			srv.join();
		} else {
			System.out.println("Usage");
			System.out.printf("   java -cp * %s listenport forwardurl_prefix targetpath_prefix concurrencylimit busyurl verbose", LimitingProxy.class);
		}
	}
}
