/*
 * Copyright 2021 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.nettool.proxy;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.eclipse.jetty.http.HttpURI;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletHandler;
import org.eclipse.jetty.servlets.ProxyServlet;

public class LogProxy extends ProxyServlet {
	private static int listenport;
	private static String forwardurl_prefix;
	private static String targetpath_prefix;
	private static PrintStream logps;
	private static int nbyt;
	@Override
	public void service(ServletRequest req, ServletResponse res) throws ServletException, IOException 
	{
	    URL target = new URL(forwardurl_prefix);
	    if(target.getPort() == target.getDefaultPort() || target.getPort() <= 0) {
	        ((Request)req).getConnection().getRequestFields().put("Host", target.getHost());
	    } else {
	        ((Request)req).getConnection().getRequestFields().put("Host", target.getHost() + ":" + target.getPort());
	    }
		synchronized(logps) { // serialize to avoid messing up log output
	    	if(((HttpServletRequest)req).getPathInfo().startsWith(targetpath_prefix)) {
	    		Logger.getLogger("org.eclipse.jetty.servlets.dk.vajhoej.nettool.proxy").setLevel(Level.FINEST);
	    	} else {
				Logger.getLogger("org.eclipse.jetty.servlets.dk.vajhoej.nettool.proxy").setLevel(Level.OFF);
	    	}
	    	nbyt = 0;
			super.service(req, res);
		}
	}
	@Override
	protected HttpURI proxyHttpURI(final String scheme, final String serverName, int serverPort, final String path) throws MalformedURLException {
		return new HttpURI(forwardurl_prefix + path);
	}
	public static void main(String[] args) throws Exception  {
		Logger.getLogger("org.eclipse.jetty.servlets.dk.vajhoej.nettool.proxy").addHandler(new Handler() {
			@Override
			public void publish(LogRecord record) {
				String line = record.getMessage().replaceAll("^\\d+ ", ""); // strip numeric prefix in lines
				if(line.startsWith("proxy ")) { // request will follow
					logps.println(">>>> Request <<<<");
				} else if(line.equals("complete")) { // done
					logps.println("");
					logps.println("(" + nbyt + " bytes)");
					logps.println(">>>> ******** <<<<");
					logps.flush();
				} else { // header or content
					if(line.startsWith("HTTP/1.1")) { // first line of response
						logps.println("");
						logps.println("(" + nbyt + " bytes)");
						nbyt = 0;
						logps.println(">>>> Response <<<<");
					}
					if(line.startsWith("content")) { // content
						nbyt += Integer.parseInt(line.substring("content".length()));
					} else { // header
						if(!line.contains("(filtered)")) { // skip headers inserted by proxy
							logps.println(line);
						}
					}
				}
			}
			@Override
			public void flush() {
				logps.flush();
			}
			@Override
			public void close() {
				logps.close();
			}
		});
		if(args.length == 4) {
			listenport = Integer.parseInt(args[0]);
			forwardurl_prefix = args[1];
			targetpath_prefix = args[2];
			logps = new PrintStream(new FileOutputStream(args[3]));
			Server srv = new Server(listenport);
	        ServletHandler proxy = new ServletHandler();
	        proxy.addServletWithMapping(LogProxy.class, "/*");
	        srv.setHandler(proxy);
			srv.start();
			srv.join();
		} else {
			System.out.println("Usage");
			System.out.printf("   java -cp * %s listenport forwardurl_prefix target_prefix logfile", LogProxy.class);
		}
	}
}
